import sys
import logging as log
from Bio import Entrez
from urllib.error import HTTPError
from .data_models import Taxon


def fetch_taxonomic_info(user_email: str, taxon: Taxon, retries: int) -> None:
    """Receives a Taxon object and tries to fetch its full taxonomic classification information

    Parameters:
    user_email (string): A valid email provided by the user and used for Entrez.email
    taxon (object): A Taxon object that will hold the fetched information and provide the input
                    information
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    Entrez.email = user_email
    Entrez.max_tries = retries
    Entrez.sleep_between_tries = 15
    # To avoid bugs, this has to be initialized as an empty dict
    taxon.classification = dict()

    try:
        query = Entrez.efetch(db="taxonomy", id=taxon.taxon_id, retmode="xml")
        parsed = Entrez.read(query)

        taxonomic_info = parsed[0]["LineageEx"]

        for taxon_level in taxonomic_info:
            if taxon_level["Rank"] == "no rank":
                if "no rank" not in list(taxon.classification):
                    taxon.classification[taxon_level["Rank"]] = []
                taxon.classification[taxon_level["Rank"]].append(
                    taxon_level["ScientificName"])
            else:
                taxon.classification[taxon_level["Rank"]
                                     ] = taxon_level["ScientificName"]
    except (KeyboardInterrupt):
        log.warning("\nQUIT: TaIGa was stopped by the user\n")
        sys.exit()
    except (HTTPError):
        log.warning("\nWARNING: Connection error while trying to fetch the taxonomic information "
                    f"for {taxon.name}. It could be due to a lack of internet connection or a broken response "
                    "from the NCBI servers. Ignoring this taxon for now")
        taxon.missing_classification = True
    except (IndexError):
        log.warning("\nWARNING: Couldn't find the taxonomic information for "
                    f"organism '{taxon.name}'")
        taxon.missing_classification = True
    except (Exception):
        log.warning("\nWARNING: Unknown error occurred while trying to fetch the taxonomic "
                    f"information for organism '{taxon.name}'. It could be due to TaIGa reaching the maximum "
                    "number of retries or issues with the NCBI servers. Maybe wait and try again a "
                    "bit later")
        taxon.missing_classification = True


def fetch_correct_spelling(user_email: str, taxon: Taxon, retries: int) -> None:
    """Receives a Taxon object and tries to fetch a correct name for it from NCBI

    Parameters:
    user_email (string): A valid email provided by the user and used for Entrez.email
    taxon (object): A Taxon object that will hold the fetched information and provide the input
                    information
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    Entrez.email = user_email
    Entrez.max_tries = retries
    Entrez.sleep_between_tries = 15

    try:
        query = Entrez.espell(db="taxonomy", term=taxon.name)
        parsed = Entrez.read(query)
        corrected_name = parsed["CorrectedQuery"]

        if (len(corrected_name) == 0):
            log.warning(
                f"\nWARNING: Couldn't find the correct organism name for '{taxon.name}'")
            taxon.missing_corrected = True
        else:
            taxon.name = corrected_name
    except (KeyboardInterrupt):
        log.warning("\nQUIT: TaIGa was stopped by the user\n")
        sys.exit()
    except (RuntimeError):
        log.warning(
            f"\nWARNING: Couldn't find the correct organism name for '{taxon.name}'")
        taxon.missing_corrected = True
    except (Exception):
        log.warning("\nWARNING: Unknown error occurred while trying to correct the spelling for "
                    f"organism '{taxon.name}'")
        taxon.missing_corrected = True


def fetch_id_from_name(user_email: str, db: str, taxon: Taxon, retries: int) -> None:
    """Fetches either the Taxon ID or the Genome ID for a Taxon object, using the taxon's name

    Parameters:
    user_email (string): A valid email provided by the user and used for Entrez.email
    db (string): The database the function should try to fetch the information from. If the value
                 is 'genome', it will search Genome for a Genome ID. If it is 'taxonomy', it will
                 search Taxonomy for a Taxon ID
    taxon (object): A Taxon object that will hold the fetched information and provide the input
                    information
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    Entrez.email = user_email
    Entrez.max_tries = retries
    Entrez.sleep_between_tries = 15

    if not taxon.missing_name:
        query = Entrez.esearch(db=db, term=taxon.name, retmode="xml")
        parsed = Entrez.read(query)

        if db == "taxonomy":
            try:
                taxon_id = parsed["IdList"][0]
                taxon.taxon_id = int(taxon_id)
            except (KeyboardInterrupt):
                log.warning("\nQUIT: TaIGa was stopped by the user\n")
                sys.exit()
            except (IndexError):
                log.warning(
                    f"\nWARNING: Couldn't find a valid Taxon ID for the organism '{taxon.name}'")
                taxon.missing_taxon_id = True
            except (Exception):
                log.warning("\nWARNING: Unknown error occurred while trying to find a valid Taxon "
                            f"ID for organism '{taxon.name}'")
                taxon.missing_taxon_id = True
        elif db == "genome":
            try:
                genome_id = parsed["IdList"][-1]
                taxon.genome_id = int(genome_id)
            except (KeyboardInterrupt):
                log.warning("\nQUIT: TaIGa was stopped by the user\n")
                sys.exit()
            except (IndexError):
                log.warning(f"\nWARNING: Couldn't find a Genome ID for oragnism '{taxon.name}'. It probably "
                            "doesn't have one available on NCBI")
            except (NameError):
                log.warning(f"\nWARNING: No Genome ID found for organism '{taxon.taxon_id}'. TaIGa probably "
                            "didn't find the organism name for this Taxon ID")
            except (Exception):
                log.warning("\nWARNING: An unknown error occurred while searching Taxonomy for the "
                            f"Genome ID of '{taxon.taxon_id}'")
    else:
        log.warning(
            f"\nWARNING: Taxon {taxon.taxon_id} is missing a name. Not searching for Taxon or Genome ID")


def fetch_name_from_taxon_id(user_email: str, taxon: Taxon, retries: int) -> None:
    """Receives a Taxon object and tries to fetch a name for it from its Taxon ID

    Parameters:
    user_email (string): A valid email provided by the user and used for Entrez.email
    taxon (object): A Taxon object that will hold the fetched information and provide the input
                    information
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    Entrez.email = user_email
    Entrez.max_tries = retries
    Entrez.sleep_between_tries = 15

    try:
        query = Entrez.efetch(db="taxonomy", id=taxon.taxon_id, retmode="xml")
        parsed = Entrez.read(query)
        name = parsed[0]["ScientificName"]

        taxon.name = name
    except (KeyboardInterrupt):
        log.warning("\nQUIT: TaIGa was stopped by the user\n")
        sys.exit()
    except (IndexError):
        log.warning(
            f"\nWARNING: Couldn't find an organism name for '{taxon.taxon_id}'")
        taxon.missing_name = True
    except (Exception):
        log.warning("\nWARNING: An unknown error occurred while searching Taxonomy for the name of "
                    f"organism '{taxon.taxon_id}'")
        taxon.missing_name = True



import logging as log


def config_log(create_log: bool) -> None:
    """Sets the global configuration for the output mode of the 'logging' module

    Parameters:
    create_log (boolean): Sets TaIGa's verbosity mode. If True, TaIGa will not be verbose

    Returns:
    None

    """

    if create_log:
        log.basicConfig(filename="TaIGa_run.log",
                        format="%(message)s", level=log.DEBUG)
    else:
        log.basicConfig(format="%(message)s", level=log.DEBUG)



import sys
import logging as log
from ..common import parsers, helpers, retrievers, data_handlers


def run_taiga(infile: str,
              outdir: str,
              email: str,
              gb_mode: int = 0,
              tid: bool = False,
              correction: bool = False,
              retries: int = 5,
              silent: bool = False) -> None:
    """ Wrapper for all of TaIGa's main functionalities

    Required Parameters:
    infile (str): Full path to input file
    outdir (str): Full path to output directory, wich doesn't need to be pre-existant
    email (str): Valid user e-mail

    Optional Parameters:
    gb_mode (int): An integer representing the reading mode for the input file. Options are:
                   0: a plain text file, which is not handled by this function (Default)
                   1: Genbank file with multiple records from different organisms
                   2: Genbank file with a single record from a single organism
                   3: Genbank file with multiple records from the same organism
    tid (bool: Default = False): Tells if TaIGa should expect a input file with Taxon IDs
    correction (bool: Default = False): Activates the name correction function.
    retries (int: Default = 5): How many time TaIGa should retry after a bad response.
    silent (bool: Default = False): Tells if TaIGa should stop printing and create a log file

    Returns:
    None

    """

    # Ouput and input paths
    input_path = infile
    if outdir[-1] == "/":
        output_path = outdir
    else:  # Adding a trailing forward slash to the output path if needed
        output_path = outdir + "/"

    # Providing the email when doing requests through E-Utils is recommended
    user_email = email

    # Minor config variables for some of TaIGa's functionalities
    retries = retries
    create_log = silent
    name_correction = correction

    # The switches for TaIGa's execution modes, either for Taxon IDs or Genbank files
    taxid = tid
    mode = gb_mode

    # A list to hold Taxon objects
    taxon_list = []

    # Inital configuration for the logging module
    # At this point, the output may be set to verbose or not
    helpers.config_log(create_log)

    log.info("""
    *********************************************
    *                                           *
    *   TaIGa - Taxonomy Information Gatherer   *
    *                                           *
    *********************************************""")

    # Checking if TaIGa is being run on Taxon ID mode with the '-c' argument
    # This is needed because, when run with '--tid', TaIGa never actually tries to correct spelling
    # as the retrieved name is assumed to be correct
    if taxid and name_correction:
        log.error("\nERROR: Please, when running TaIGa with the '--tid' option, don't use the '-c' "
                  "option as TaIGa already skips the name correction\n")
        sys.exit()

    # Check if input mode is for a Genbank format file or a text file and then parse the input
    if not (mode == 0):
        taxon_list = parsers.parse_gb(input_path, mode)
    else:
        taxon_list = parsers.parse_txt(input_path, taxid)

    log.info("\n> Searching for taxonomic information...")

    # Checking the type of input (Taxon ID or names) and fetching the rest of the information
    if tid:
        retrievers.retrieve_from_taxid(taxon_list, user_email, retries)
    else:
        retrievers.retrieve_from_names(
            taxon_list, user_email, name_correction, retries)

    # Calling the wrapper function to fetch for the taxonomic information for all organisms
    retrievers.retrieve_taxonomy(taxon_list, user_email, retries)

    # Calling a function to handle the fetched data and convert it to a Pandas DataFrame
    frame = data_handlers.create_df(taxon_list)
    # Calling the last function which takes the DataFrame and creates the output files
    data_handlers.create_output(output_path, frame, taxon_list)

    log.info(
        "\n> TaIGa was run successfully! You can check the results inside the output folder\n")



import sys
import logging as log
from Bio import SeqIO
from .data_models import Taxon
from typing import List, Set


def parse_txt(input_path: str, tid: bool) -> List:
    """Parses a file containing multiple strings and return a list of objects

    Parameters:
    input_path (string): The path to the input file as a string
    tid (boolean): A variable informing if the input file contains Taxon IDs

    Returns:
    list_of_taxa (list): A list of Taxon objects, each containing a self.name value for each name
                         in the input file or a self.tid value for each Taxon ID in the input file

    """

    list_of_taxa: List = []

    try:

        with open(input_path, "r") as infile:
            # Using set to uniquefy possible duplicate names or IDs from the input
            all_inputs: Set = set(infile.readlines())

            # Checks first if input file is an ids or names file
            if tid:
                log.info("\n> Parsing input file a simple list of Taxon IDs\n")

                for each_id in all_inputs:
                    # Each taxon might contain a '\n' character at the end, so remove it
                    new_id = Taxon(taxon_id=int(each_id.replace("\n", "")))

                    list_of_taxa.append(new_id)

                    log.info(f"{new_id.taxon_id} ---> All OK")
            else:
                log.info("\n> Parsing input file a simple list of species names\n")

                for each_taxon in all_inputs:
                    # Each taxon might contain a '\n' character at the end, so remove it
                    new_taxon = Taxon(name=each_taxon.replace("\n", ""))

                    list_of_taxa.append(new_taxon)

                    log.info(f"{new_taxon.name} ---> All OK")
    except (KeyboardInterrupt):
        log.warning("\nQUIT: TaIGa was stopped by the user\n")
        sys.exit()
    except (Exception):
        log.error(
            "\nERROR: Couldn't parse input text file. Check your file an try running TaIGa again\n"
        )
        sys.exit()

    return list_of_taxa


def parse_gb(input_path: str, mode: int) -> List:
    """Parses a Genbank format file containing sequence records and organism information

    Parameters:
    input_path (string): The path to the input file as a string
    mode (int): An integer representing the reading mode for the input file. Options are:
                0: a plain text file, which is not handled by this function
                1: Genbank file with multiple records from different organisms
                2: Genbank file with a single record from a single organism
                3: Genbank file with multiple records from the same organism

    Returns:
    list_of_taxa (list): A list of Taxon objects, each containing a self.name value for each record
                  in the input file

    """

    list_of_taxa: List = []

    try:
        if mode == 1:
            log.info(
                "\n> Parsing input as a Genbank file with multiple records and organisms")

            input_records = SeqIO.parse(input_path, "genbank")

            # Using a set to uniquefy possible duplicate names or IDs from the input
            list_of_taxa = list(
                {Taxon(name=seq.annotations["organism"]) for seq in input_records})
        elif mode == 2:
            log.info("\n> Parsing input as a Genbank file with a single record")
            try:
                input_records = SeqIO.read(input_path, "genbank")

                list_of_taxa.append(
                    Taxon(name=input_records.annotations["organism"]))
            except (ValueError):
                # Catch an error if there's more than one record in the input file.
                log.error("\nERROR: The file contains more than one record. Try running TaIGa "
                          "again with a different Genbank file mode\n")
                sys.exit()
        elif mode == 3:
            log.info(
                "\n> Parsing input as a Genbank file with multiple records for one organism")

            input_records = SeqIO.parse(input_path, "genbank")

            for record in input_records:
                # Only take the first item of the generator. Is faster than conveting to a list.
                list_of_taxa.append(Taxon(name=record.annotations["organism"]))
                break
    except (KeyboardInterrupt):
        log.warning("\nQUIT: TaIGa was stopped by the user.\n")
        sys.exit()
    except(Exception):
        log.error(
            "\nERROR: Couldn't parse input genome file. Check your file and try running TaIGa "
            "again.\n")
        sys.exit()

    if list_of_taxa:
        log.info("\n> All OK with parsing the input file. Checking the records...\n")
        for taxon in list_of_taxa:
            log.info(f"> '{taxon.name}' ---> All OK")
    else:
        log.error("\nERROR: Something went wrong while trying to parse the input file. Check the "
                  "file and the program execution commands and try again\n")
        sys.exit()

    return list_of_taxa



import pandas as pd
import logging as log
import sys
import os
from typing import List, Set


def create_df(taxon_list: List) -> pd.DataFrame:
    """Creates a Pandas DataFrame with the information for each input taxon

    Parameters:
    taxon_list (list): A list with all the Taxa the will provide the input information for the
                       DataFrame

    Returns:
    frame (DataFrame): A Pandas DataFrame object with all the taxonomic information organized for
                       each taxon

    """

    log.info("\n> Generating result DataFrame")

    # Headers set, will store all possible ranks from the input taxa
    raw_ranks: Set = set()

    # The header set is a combination of the rank sets from all taxa
    for taxon in taxon_list:
        if not (taxon.missing_name or taxon.missing_taxon_id or taxon.missing_classification):
            raw_ranks |= taxon.list_ranks()

    # To preserve the order, a list with all possible ranks from NCBI taxonomy is constructed
    ordered_ranks: List = [
        "no rank", "superkingdom", "kingdom", "subkingdom", "phylum", "subphylum",
        "superclass", "class", "subclass", "infraclass", "cohort", "subcohort",
        "superorder", "order", "suborder", "infraorder", "parvorder", "superfamily",
        "family", "subfamily", "tribe", "subtribe", "genus", "subgenus", "section",
        "subsection", "series", "species-group", "species", "subspecies", "forma"
    ]

    # To assure only ranks existing in the input taxa are part of the header, filter the list
    final_ranks: List = [rank for rank in ordered_ranks if rank in raw_ranks]

    # Add final header variables
    final_ranks.insert(0, "genome_id")
    final_ranks.insert(0, "taxon_id")

    # Create lists from the names and classifications of each taxon, in order
    taxon_names: List = [taxon.name for taxon in taxon_list if not (taxon.missing_name or
                                                                    taxon.missing_taxon_id or
                                                                    taxon.missing_classification)]
    taxon_classification: List = [taxon.classification for taxon in taxon_list
                                  if not (taxon.missing_name or taxon.missing_taxon_id
                                          or taxon.missing_classification)]

    # Create a dataframe from the lists of classifications, names and ranks
    frame: pd.DataFrame = pd.DataFrame(taxon_classification,
                                       index=taxon_names, columns=final_ranks)

    # Add the values for Taxon ID and Genome ID for each taxon
    for taxon in taxon_list:
        if not (taxon.missing_name or taxon.missing_taxon_id or taxon.missing_classification):
            frame.at[taxon.name, "taxon_id"] = taxon.taxon_id
            frame.at[taxon.name, "genome_id"] = taxon.genome_id

    # Convert the Taxon IDs to integers
    frame.taxon_id = frame.taxon_id.astype(int)

    # Change all 'NaN' occurences for 'N/A'
    frame.fillna("N/A", inplace=True)

    log.info("\n> Done generating result DataFrame")

    return frame


def create_output(output_path: str, frame: pd.DataFrame, taxon_list: List) -> None:
    """Creates the output directories (if they don't exist) and files for TaIGa

    Parameters:
    output_path (string): The path to the output folder as a string. It doesn't need to exist
    frame (DataFrame): The DataFrame containing all the information for all Taxa
    taxon_list (list): A list of Taxa objects used to create the file with the missing information

    Returns:
    None

    """
    log.info("\n> Checking if output folder exists")

    if not os.path.exists(output_path):
        log.info("\n> Creating output folder")

        try:
            os.makedirs(output_path)
        except (Exception):
            log.error(
                "\nERROR: Path to output folder may not be valid. Try again\n")
            sys.exit()

    log.info(
        "\n> Creating output file. You'll find it inside the provided output folder")

    # Export the DataFrame to the resulting .csv file
    frame.to_csv(output_path + "TaIGa_result.csv")

    log.info(
        "\n> Creating a file for the organisms with missing information. It might be empty")

    with open(output_path + "TaIGa_missing.txt", "w") as missing_file:
        missing_file.write("Missing corrected name: \n")
        for taxon in taxon_list:
            if taxon.missing_corrected:
                missing_file.write(f"\t{taxon.name}\n")

        missing_file.write("Missing Taxon ID: \n")
        for taxon in taxon_list:
            if taxon.missing_taxon_id:
                missing_file.write(f"\t{taxon.name}\n")

        missing_file.write("Missing name: \n")
        for taxon in taxon_list:
            if taxon.missing_name:
                missing_file.write(f"\t{taxon.taxon_id}\n")

        missing_file.write("Missing classification: \n")
        for taxon in taxon_list:
            if taxon.missing_classification:
                if not taxon.missing_name:
                    missing_file.write(f"\t{taxon.name}\n")
                else:
                    missing_file.write(f"\t{taxon.taxon_id}\n")



import sys
import logging as log
from . import fetchers
from typing import List


def retrieve_from_taxid(taxon_list: List, user_email: str, retries: int) -> None:
    """Wrapper for the functions that get the name and Genome ID for each taxon from their Taxon ID.
    It writes the fetched information directly in the Taxon objects provided

    Parameters:
    taxon_list (list): A list containing all Taxon objects parsed from the input file
    user_email (string): A valid email address provided for the user and used for Entrez.email
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    for each_taxon in taxon_list:
        try:
            log.info("\n> Searching for corresponding organism name and Genome ID of "
                     f"'{each_taxon.taxon_id}'")

            fetchers.fetch_name_from_taxon_id(user_email, each_taxon, retries)

            log.info(
                f" >>>> Name for '{each_taxon.taxon_id}': '{each_taxon.name}'")

            fetchers.fetch_id_from_name(
                user_email, "genome", each_taxon, retries)

            log.info(
                f" >>>> Genome ID for '{each_taxon.taxon_id}': '{each_taxon.genome_id}'")
        except (KeyboardInterrupt):
            log.warning("\nQUIT: TaIGa was stopped by the user\n")
            sys.exit()


def retrieve_from_names(taxon_list: List, user_email: str, correction: bool, retries: int) -> None:
    """Wrapper for the functions that get the Taxon ID and Genome ID for each taxon from their name.
    It writes the fetched information directly in the Taxon objects provided

    Parameters:
    taxon_list (list): A list containing all Taxon objects parsed from the input file
    user_email (string): A valid email address provided for the user and used for Entrez.email
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt
    correction (bool): Tells the function if it should run the spell correction function on the
                       taxon names before trying to fetch the other information

    Returns:
    None

    """

    for each_taxon in taxon_list:
        try:
            if correction:
                log.info(
                    f"\n> Correcting organism name of '{each_taxon.name}'")

                fetchers.fetch_correct_spelling(
                    user_email, each_taxon, retries)

            log.info(
                f"\n> Searching Taxon ID and Genome ID of organism '{each_taxon.name}'")

            fetchers.fetch_id_from_name(
                user_email, "taxonomy", each_taxon, retries)

            log.info(
                f" >>>> Taxon ID for '{each_taxon.name}': '{each_taxon.taxon_id}'")

            fetchers.fetch_id_from_name(
                user_email, "genome", each_taxon, retries)

            log.info(
                f" >>>> Genome ID for '{each_taxon.name}': '{each_taxon.genome_id}'")
        except (KeyboardInterrupt):
            log.warning("\nQUIT: TaIGa was stopped by the user\n")
            sys.exit()


def retrieve_taxonomy(taxon_list: List, user_email: str, retries: int) -> None:
    """Wrapper for the function that fetches a taxon's taxonomic classification information

    Parameters:
    taxon_list (list): A list containing all Taxon objects parsed from the input file
    user_email (string): A valid email address provided for the user and used for Entrez.email
    retries (int): The maximum number of retries after an unsuccsessful fetch attempt

    Returns:
    None

    """

    log.info(
        "\n> Gathering taxonomic information from NCBI Taxonomy. This might take a while...")

    for each_taxon in taxon_list:
        if not (each_taxon.missing_name or each_taxon.missing_taxon_id):
            try:
                fetchers.fetch_taxonomic_info(user_email, each_taxon, retries)
            except (KeyboardInterrupt):
                log.warning("\nQUIT: TaIGa was stopped by the user\n")
                sys.exit()
        elif each_taxon.missing_name:
            log.warning(
                f"\nWARNING: Taxon {each_taxon.taxon_id} is missing critical information. Skipping it...")
        elif each_taxon.missing_taxon_id:
            log.warning(
                f"\nWARNING: Taxon {each_taxon.name} is missing critical information. Skipping it...")

    log.info("\n> Done gathering taxonomic information")



"""
A Taxon holds all the information parsed an fetched by TaIGa for one organism
"""
from typing import Dict, List, Set


class Taxon:
    def __init__(self, name: str = None, genome_id: int = None, taxon_id: int = None, classification: Dict = dict(),
                 missing_name: bool = False, missing_taxon_id: bool = False, missing_corrected: bool = False,
                 missing_classification: bool = False) -> None:
        self.name = name
        self.genome_id = genome_id
        self.taxon_id = taxon_id
        self.classification = classification
        self.missing_name = missing_name
        self.missing_taxon_id = missing_taxon_id
        self.missing_corrected = missing_corrected
        self.missing_classification = missing_classification

    def list_taxa(self) -> List:
        """Returns a list of all the names of the taxon ranks for an organism

        Parameters:
        self (object): A reference to the object itself

        Returns:
        (list): A list of all the values from the 'self.classification' dict

        """

        return list(self.classification.values())

    def list_ranks(self) -> Set:
        """Returns a set with all the ranks for an organism

        Parameters:
        self (object): A reference to the object itself

        Returns:
        (set): A set of all the unique keys from the 'self.classification' dict

        """

        return set(self.classification)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="taiga-bio",
    version="1.0.2",
    author="Maycon Oliveira",
    author_email="flayner5@gmail.com",
    description="Set of tools to fetch taxonomic metadata for a list of organisms",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/flayner2/taiga-bio",
    packages=setuptools.find_packages(),
    install_requires=["biopython", "pandas"],
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Scientific/Engineering :: Bio-Informatics"],
    python_requires=">=3.6"
)
